// Java class for specification of track segments
public class Segment extends RailwayNetwork {

    Segment leftNeighbouringSegment;
    Segment rightNeighbouringSegment;

    String segmentId;
    int    segmentLength;
    Train  currentTrain;

    Segment(String id, int length) {
        segmentId = id;
	segmentLength = ( length == 0 ) ? SHORT : LONG;
        leftNeighbouringSegment = null;
        rightNeighbouringSegment = null;
        currentTrain = null;
    }

    Segment(String id, Segment left, Segment right, int length) {

        segmentId = id;
	segmentLength = ( length == 0 ) ? SHORT : LONG;
        leftNeighbouringSegment = left;
        rightNeighbouringSegment = right;
        currentTrain = null;

    }

    public String toString() { return segmentId; }
    public boolean isShort() { return segmentLength == SHORT; }

    public synchronized Segment getNeighbour(int direction) {
	return ( direction == LEFT ) 
            ? leftNeighbouringSegment 
	    : rightNeighbouringSegment;
    }

    public synchronized Train getTrainOnSegment() { return currentTrain; }

    public synchronized void enterSegment(Train tr) throws Exception { 
     
        if ( currentTrain == null ) 
            currentTrain = tr;
        else {
            throw new 
               Exception("Two trains on Segment " 
                          + toString()
                          +": "
                          + currentTrain.toString()
                          + " and "
                          + tr.toString()
                          + "\n");
	} 

    }

    public synchronized void leaveSegment(Train tr) { 
	if ( tr == currentTrain) currentTrain = null;
    }

    public synchronized void setNeighbour(int dir, Segment seg) {
	switch(dir) {
	    case LEFT: 
		leftNeighbouringSegment = seg; 
		System.out.println(toString() 
				   + ": LINKED LEFT TO "
				   + leftNeighbouringSegment.toString()
				   + "\n");
		break; 
	    case RIGHT: 
		rightNeighbouringSegment = seg; 
		System.out.println(toString() 
				   + ": LINKED RIGHT TO "
				   + rightNeighbouringSegment.toString()
				   + "\n");
		break;
	}
    }

}
